/**********************************************************************
 *
 * ast_sock_cmd: Pipe commands to asterisk through the asterisk control
 *               socket (asterisk.ctl)
 *
 * Written by Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 * Copyright (C) 2005, Xorcom
 *
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ********************************************************************
 * Asterisk listens on that socket for commands which should be 
 * terminated with \r\n . The input is sent back in the same connection.
 *
 * This program reads commands from its standard input and sends them
 * to that socket (with \r\n). It does not attempt to read the output 
 * from the socket and thus will not give any indication of succes or 
 * failure.
 *
 * Unlike asterisk -r it starts much faster and can be piped multiple 
 * commands from its standard input.
 ********************************************************************
 */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdio.h>
#include <errno.h>

#define MAX_CMD_LEN 80
#define SOCK_FILE "/var/run/asterisk/asterisk.ctl"

// borrowed from Asterisk's ast_tryconnect
int main () {
  char command[MAX_CMD_LEN+2];
  struct sockaddr_un sock_addr;
  int res;
  int sock = socket(PF_UNIX, SOCK_STREAM, 0);
  if (sock < 0) {
    fprintf(stderr,"Failed to create socket: %s\n", strerror(errno));
    exit(1);
  }
  memset(&sock_addr, 0, sizeof(sock_addr));
  sock_addr.sun_family = AF_UNIX;
  strcpy(sock_addr.sun_path, SOCK_FILE);
  res = connect(sock, (struct sockaddr *)&sock_addr, sizeof(sock_addr));
  if (res < 0) {
    fprintf(stderr,"Failed to connect to asterisk control socket: %s\n", strerror(errno));
    exit(2);
  }
  
  while (fgets(command, MAX_CMD_LEN, stdin)) {
    // stand at the end of the string and terminate it with \r\n\0
    char *p = strchr(command, '\n');
    if (p == NULL) { p=strchr(command,'\0');}
    p[0]='\r'; // TODO: check if there was already a \r ?
    p[0]='\n';
    p[0]='\0';

    // for the sake of simpliicty: let's assume that if write wrote, 
    // it wrote everything:
    res = write(sock, command, strlen(command));
    if (res < 0) {
      fprintf(stderr,"Failed to write to asterisk control socket: %s\n", 
          strerror(errno));
      exit(3); // any point of close()-ing here?
    }
  }
  close(sock);
}
